@@ -34,6 +34,7 @@ import numpy as np
 from mlagents_envs.exception import UnityActionException
 
 AgentId = int
+GroupId = int
 BehaviorName = str
 
 
@@ -56,6 +57,8 @@ class DecisionStep(NamedTuple):
     reward: float
     agent_id: AgentId
     action_mask: Optional[List[np.ndarray]]
+    group_id: int
+    group_reward: float
 
 
 class DecisionSteps(Mapping):
@@ -81,11 +84,13 @@ class DecisionSteps(Mapping):
      this simulation step.
     """
 
-    def __init__(self, obs, reward, agent_id, action_mask):
+    def __init__(self, obs, reward, agent_id, action_mask, group_id, group_reward):
         self.obs: List[np.ndarray] = obs
         self.reward: np.ndarray = reward
         self.agent_id: np.ndarray = agent_id
         self.action_mask: Optional[List[np.ndarray]] = action_mask
+        self.group_id: np.ndarray = group_id
+        self.group_reward: np.ndarray = group_reward
         self._agent_id_to_index: Optional[Dict[AgentId, int]] = None
 
     @property
@@ -120,11 +125,14 @@ class DecisionSteps(Mapping):
             agent_mask = []
             for mask in self.action_mask:
                 agent_mask.append(mask[agent_index])
+        group_id = self.group_id[agent_index]
         return DecisionStep(
             obs=agent_obs,
             reward=self.reward[agent_index],
             agent_id=agent_id,
             action_mask=agent_mask,
+            group_id=group_id,
+            group_reward=self.group_reward[agent_index],
         )
 
     def __iter__(self) -> Iterator[Any]:
@@ -144,6 +152,8 @@ class DecisionSteps(Mapping):
             reward=np.zeros(0, dtype=np.float32),
             agent_id=np.zeros(0, dtype=np.int32),
             action_mask=None,
+            group_id=np.zeros(0, dtype=np.int32),
+            group_reward=np.zeros(0, dtype=np.float32),
         )
 
 
@@ -163,6 +173,8 @@ class TerminalStep(NamedTuple):
     reward: float
     interrupted: bool
     agent_id: AgentId
+    group_id: GroupId
+    group_reward: float
 
 
 class TerminalSteps(Mapping):
@@ -183,11 +195,13 @@ class TerminalSteps(Mapping):
      across simulation steps.
     """
 
-    def __init__(self, obs, reward, interrupted, agent_id):
+    def __init__(self, obs, reward, interrupted, agent_id, group_id, group_reward):
         self.obs: List[np.ndarray] = obs
         self.reward: np.ndarray = reward
         self.interrupted: np.ndarray = interrupted
         self.agent_id: np.ndarray = agent_id
+        self.group_id: np.ndarray = group_id
+        self.group_reward: np.ndarray = group_reward
         self._agent_id_to_index: Optional[Dict[AgentId, int]] = None
 
     @property
@@ -218,11 +232,14 @@ class TerminalSteps(Mapping):
         agent_obs = []
         for batched_obs in self.obs:
             agent_obs.append(batched_obs[agent_index])
+        group_id = self.group_id[agent_index]
         return TerminalStep(
             obs=agent_obs,
             reward=self.reward[agent_index],
             interrupted=self.interrupted[agent_index],
             agent_id=agent_id,
+            group_id=group_id,
+            group_reward=self.group_reward[agent_index],
         )
 
     def __iter__(self) -> Iterator[Any]:
@@ -242,6 +259,8 @@ class TerminalSteps(Mapping):
             reward=np.zeros(0, dtype=np.float32),
             interrupted=np.zeros(0, dtype=np.bool),
             agent_id=np.zeros(0, dtype=np.int32),
+            group_id=np.zeros(0, dtype=np.int32),
+            group_reward=np.zeros(0, dtype=np.float32),
         )
 
 
@@ -391,28 +410,20 @@ class ActionSpec(NamedTuple):
         return ActionTuple(continuous=_continuous, discrete=_discrete)
 
     def _validate_action(
-        self, actions: ActionTuple, n_agents: Optional[int], name: str
+        self, actions: ActionTuple, n_agents: int, name: str
     ) -> ActionTuple:
         """
         Validates that action has the correct action dim
         for the correct number of agents and ensures the type.
         """
-        _expected_shape = (
-            (n_agents, self.continuous_size)
-            if n_agents is not None
-            else (self.continuous_size,)
-        )
+        _expected_shape = (n_agents, self.continuous_size)
         if actions.continuous.shape != _expected_shape:
             raise UnityActionException(
                 f"The behavior {name} needs a continuous input of dimension "
                 f"{_expected_shape} for (<number of agents>, <action size>) but "
                 f"received input of dimension {actions.continuous.shape}"
             )
-        _expected_shape = (
-            (n_agents, self.discrete_size)
-            if n_agents is not None
-            else (self.discrete_size,)
-        )
+        _expected_shape = (n_agents, self.discrete_size)
         if actions.discrete.shape != _expected_shape:
             raise UnityActionException(
                 f"The behavior {name} needs a discrete input of dimension "
@@ -488,6 +499,10 @@ class ObservationSpec(NamedTuple):
     dimension_property: Tuple[DimensionProperty, ...]
     observation_type: ObservationType
 
+    # Optional name. For observations coming from com.unity.ml-agents, this
+    # will be the ISensor name.
+    name: str
+
 
 class BehaviorSpec(NamedTuple):
     """
