@@ -63,7 +63,8 @@ class UnityEnvironment(BaseEnv):
     #  * 1.2.0 - support compression mapping for stacked compressed observations.
     #  * 1.3.0 - support action spaces with both continuous and discrete actions.
     #  * 1.4.0 - support training analytics sent from python trainer to the editor.
-    API_VERSION = "1.4.0"
+    #  * 1.5.0 - support variable length observation training and multi-agent groups.
+    API_VERSION = "1.5.0"
 
     # Default port that the editor listens on. If an environment executable
     # isn't specified, this port will be used.
@@ -122,6 +123,8 @@ class UnityEnvironment(BaseEnv):
         capabilities.compressedChannelMapping = True
         capabilities.hybridActions = True
         capabilities.trainingAnalytics = True
+        capabilities.variableLengthObservation = True
+        capabilities.multiAgentGroups = True
         return capabilities
 
     @staticmethod
@@ -362,9 +365,9 @@ class UnityEnvironment(BaseEnv):
         if behavior_name not in self._env_state:
             return
         action_spec = self._env_specs[behavior_name].action_spec
-        num_agents = len(self._env_state[behavior_name][0])
-        action = action_spec._validate_action(action, None, behavior_name)
+        action = action_spec._validate_action(action, 1, behavior_name)
         if behavior_name not in self._env_actions:
+            num_agents = len(self._env_state[behavior_name][0])
             self._env_actions[behavior_name] = action_spec.empty_action(num_agents)
         try:
             index = np.where(self._env_state[behavior_name][0].agent_id == agent_id)[0][
